/*
 * This file is part of X2C. http://www.mechatronic-simulation.org/
 * $LastChangedRevision: 457 $
 */
/* USERCODE-BEGIN:Description                                                                                         */
/**     Description:	Continuously differentiable output change rate limit-**/
/**						ation with	sin� shape								 **/
/**						Calculation:                               			 **/
/**							du/dt = 0:										 **/
/**								y = u										 **/
/**							du/dt <> 0:										 **/
/**								y = u * sin�(t)		(simplified)			 **/
/**																			 **/
/**		Caution:	A step in the input signal during a	running limitation   **/
/**					process will start a new limitation process and the out- **/
/**					put signal won't be continuously differentiable anymore! **/
/**																			 **/
/* USERCODE-END:Description                                                                                           */
#include "Sin2Limiter_FiP32.h"

/* all used update functions to ram for c2000                                                                         */
#if defined(__ALL_UPDATE_FUNC_2_RAM_C2000__)
     #pragma CODE_SECTION(Sin2Limiter_FiP32_Update, "ramfuncs")
#endif

/* USERCODE-BEGIN:PreProcessor                                                                                        */
#include "Sin2_Data.h"

/* Inputs */
#define IN 					(*pTSin2Limiter_FiP32->In)

/* Outputs */
#define OUT 				(pTSin2Limiter_FiP32->Out)

/* Parameters */
#define RATE_UP				(pTSin2Limiter_FiP32->RateUp)			/* rising slew rate */
#define RATE_DOWN			(pTSin2Limiter_FiP32->RateDown)			/* falling slew rate */

/* Variables */
#define SCALED_RATE_UP		(pTSin2Limiter_FiP32->Scaled_RateUp)	/* to step height adjusted rising slew rate */
#define SCALED_RATE_DOWN	(pTSin2Limiter_FiP32->Scaled_RateDown)	/* to step height adjusted falling slew rate */
#define IN_OLD				(pTSin2Limiter_FiP32->In_Old)			/* input value from previous cycle */
#define LEVEL	  			(pTSin2Limiter_FiP32->Level)			/* current input level (argument) for sin� calculation */
#define STEP_HEIGHT			(pTSin2Limiter_FiP32->Step_Height)		/* current step height */
#define STATE				(pTSin2Limiter_FiP32->State)			/* 1 -> limiting at rising flank, -1 -> limiting at falling flank, 0 -> no limiting */

/* USERCODE-END:PreProcessor                                                                                          */

/**********************************************************************************************************************/
/** Update                                                                                                           **/
/**********************************************************************************************************************/
void Sin2Limiter_FiP32_Update(SIN2LIMITER_FIP32 *pTSin2Limiter_FiP32)
{
/* USERCODE-BEGIN:UpdateFnc                                                                                           */
	int32	sin2;
 	int64	derivative, temp;


	/* derivative */
	derivative = (int64)IN - (int64)IN_OLD;
	IN_OLD = IN;

	/* check for change in input signal */
	if (derivative > 0)			/* positive step */
	{						
		STATE = 1;							/* save direction information */
		LEVEL = UINT32_MAX;					/* preset level */
		STEP_HEIGHT = (uint32)derivative;	/* save step height */

		/* justification of rate factor in respect to step height */
		if (RATE_UP >= STEP_HEIGHT)	/* result is not in 0..1 range */
		{
			SCALED_RATE_UP = INT32_MAX;		/* positive saturation */
		}
		else						/* result is in 0..1 range */
		{
			SCALED_RATE_UP = (((int64)RATE_UP)<<32) / STEP_HEIGHT;
		}
	}
	else
	{
		if (derivative < 0)		/* negative step */
		{
			STATE = -1;									/* save direction information */
			LEVEL = UINT32_MAX;							/* preset level */
			STEP_HEIGHT = (uint32)(-derivative);		/* save step height */

			/* justification of rate factor in respect to step height */
			if (RATE_DOWN >= STEP_HEIGHT)	/* result is not in -1..0 range */
			{
				SCALED_RATE_DOWN = INT32_MAX;	/* negative saturation */
			}
			else							/* result is in -1..0 range */
			{
				SCALED_RATE_DOWN = (((int64)RATE_DOWN)<<32) / STEP_HEIGHT;
			}
		}
	}

	/* check for rate limitation in progress */
	if ((STATE == 0) && (derivative == 0))	/* no current rate limiting */
	{
		OUT = IN;
	}
	else									/* rate limiting is in progress */
	{
		if (STATE > 0)			/* rising flank in progress */
		{
			if (LEVEL <= SCALED_RATE_UP)   /* check if level has reached final value */
			{
				/* rate limitation has finished */
				LEVEL = (uint32)0;
				STATE = 0;
			}
			else
			{
				LEVEL -= SCALED_RATE_UP;	/* calculate new level value */
			}
			LOOKUP32(Sin2_Table32, LEVEL, sin2);	/* calculate sin�(level) */
		}
		else					/* falling flank in progress */
		{						
			if (LEVEL <= SCALED_RATE_DOWN)	   /* check if level has reached final value */
			{
				/* rate limitation has finished */
				LEVEL = (uint32)0;
				STATE = 0;
			}
			else
			{
				LEVEL -= SCALED_RATE_DOWN;	/* calculate new level value */
			}
			LOOKUP32(Sin2_Table32, LEVEL, sin2);	/* calculate sin�(level) */
			sin2 = -sin2;							/* adapt sin� to falling flank */
		}
															
		temp = ((int64)sin2 * (int64)STEP_HEIGHT) << 1;	/* scale sin� to step height */
		OUT  = IN - (int32)(temp >> 32);				/* subtract sin� from input */
	}

/* USERCODE-END:UpdateFnc                                                                                             */

}

/**********************************************************************************************************************/
/** Initialization                                                                                                   **/
/**********************************************************************************************************************/
void Sin2Limiter_FiP32_Init(SIN2LIMITER_FIP32 *pTSin2Limiter_FiP32)
{
     pTSin2Limiter_FiP32->ID = SIN2LIMITER_FIP32_ID;
     pTSin2Limiter_FiP32->Out = 0;
/* USERCODE-BEGIN:InitFnc                                                                                             */
    SCALED_RATE_UP   = 0;
    SCALED_RATE_DOWN = 0;
	IN_OLD = 0;
	LEVEL = 0;
	STEP_HEIGHT = 0;
	STATE = 0;


/* USERCODE-END:InitFnc                                                                                               */
}

/**********************************************************************************************************************/
/** Load block data                                                                                                  **/
/**********************************************************************************************************************/
uint8 Sin2Limiter_FiP32_Load(const SIN2LIMITER_FIP32 *pTSin2Limiter_FiP32, uint8 data[])
{
     uint8 loadSize;

     data[0] = (uint8)(pTSin2Limiter_FiP32->RateUp & 0x000000FF);
     data[1] = (uint8)((pTSin2Limiter_FiP32->RateUp >> 8) & 0x000000FF);
     data[2] = (uint8)((pTSin2Limiter_FiP32->RateUp >> 16) & 0x000000FF);
     data[3] = (uint8)((pTSin2Limiter_FiP32->RateUp >> 24) & 0x000000FF);
     data[4] = (uint8)(pTSin2Limiter_FiP32->RateDown & 0x000000FF);
     data[5] = (uint8)((pTSin2Limiter_FiP32->RateDown >> 8) & 0x000000FF);
     data[6] = (uint8)((pTSin2Limiter_FiP32->RateDown >> 16) & 0x000000FF);
     data[7] = (uint8)((pTSin2Limiter_FiP32->RateDown >> 24) & 0x000000FF);
     data[8] = (uint8)(pTSin2Limiter_FiP32->Scaled_RateUp & 0x000000FF);
     data[9] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateUp >> 8) & 0x000000FF);
     data[10] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateUp >> 16) & 0x000000FF);
     data[11] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateUp >> 24) & 0x000000FF);
     data[12] = (uint8)(pTSin2Limiter_FiP32->Scaled_RateDown & 0x000000FF);
     data[13] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateDown >> 8) & 0x000000FF);
     data[14] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateDown >> 16) & 0x000000FF);
     data[15] = (uint8)((pTSin2Limiter_FiP32->Scaled_RateDown >> 24) & 0x000000FF);
     data[16] = (uint8)(pTSin2Limiter_FiP32->In_Old & 0x000000FF);
     data[17] = (uint8)((pTSin2Limiter_FiP32->In_Old >> 8) & 0x000000FF);
     data[18] = (uint8)((pTSin2Limiter_FiP32->In_Old >> 16) & 0x000000FF);
     data[19] = (uint8)((pTSin2Limiter_FiP32->In_Old >> 24) & 0x000000FF);
     data[20] = (uint8)(pTSin2Limiter_FiP32->Level & 0x000000FF);
     data[21] = (uint8)((pTSin2Limiter_FiP32->Level >> 8) & 0x000000FF);
     data[22] = (uint8)((pTSin2Limiter_FiP32->Level >> 16) & 0x000000FF);
     data[23] = (uint8)((pTSin2Limiter_FiP32->Level >> 24) & 0x000000FF);
     data[24] = (uint8)(pTSin2Limiter_FiP32->Step_Height & 0x000000FF);
     data[25] = (uint8)((pTSin2Limiter_FiP32->Step_Height >> 8) & 0x000000FF);
     data[26] = (uint8)((pTSin2Limiter_FiP32->Step_Height >> 16) & 0x000000FF);
     data[27] = (uint8)((pTSin2Limiter_FiP32->Step_Height >> 24) & 0x000000FF);
     data[28] = (uint8)pTSin2Limiter_FiP32->State;
     loadSize = (uint8)29;
/* USERCODE-BEGIN:LoadFnc                                                                                             */
/* USERCODE-END:LoadFnc                                                                                               */
     return (loadSize);
}

/**********************************************************************************************************************/
/** Save block data                                                                                                  **/
/**********************************************************************************************************************/
uint8 Sin2Limiter_FiP32_Save(SIN2LIMITER_FIP32 *pTSin2Limiter_FiP32, const uint8 data[], 
          uint8 frameLength)
{
     uint8 error;

     if (frameLength != (uint8)8)
     {
          error = (uint8)1;
     }
     else
     {
          pTSin2Limiter_FiP32->RateUp = UINT32_TO_INT32((uint32)data[0] + \
               ((uint32)data[1] << 8) + ((uint32)data[2] << 16) + \
               ((uint32)data[3] << 24));
          pTSin2Limiter_FiP32->RateDown = UINT32_TO_INT32((uint32)data[4] + \
               ((uint32)data[5] << 8) + ((uint32)data[6] << 16) + \
               ((uint32)data[7] << 24));
          error = (uint8)0;
/* USERCODE-BEGIN:SaveFnc                                                                                             */
/* USERCODE-END:SaveFnc                                                                                               */
     }
     return (error);
}

/**********************************************************************************************************************/
/** Get block element address                                                                                        **/
/**********************************************************************************************************************/
#if !defined(SIN2LIMITER_FIP32_ISLINKED)
void* Sin2Limiter_FiP32_GetAddress(const SIN2LIMITER_FIP32* block, uint16 elementId)
{
    void* addr;
    switch (elementId)
    {
        case 1:
            addr = (void*)block->In;
            break;
        case 2:
            addr = (void*)&block->Out;
            break;
        default:
            addr = (void*)0;
            break;
    }
    return (addr);
}
#endif
